<template>
  <el-dialog title="临时呼叫" v-bind="$attrs" @close="onClose" @opened="onOpen" v-dialog-drag>
    <audio ref="speak" id="ptt_speak" width="320" autoplay controls muted style="display: none"></audio>
    <audio ref="listen" id="ptt_listen" width="320" autoplay controls style="display: none"></audio>
    <!-- 单人调度 -->
    <div v-if="!multiple" class="ptt-client">
      <!-- 对讲 -->
      <div class="ptt-talk">
        <div class="row">
          <!-- 调度员 -->
          <div class="info">
            <svg-icon name="ptt-platform" class="info-icon" />
            {{ sysUserInfo.name }}
            <span v-if="stream.speak">讲话中</span>
          </div>
          <!-- 被调度人 -->
          <div class="info">
            <svg-icon name="ptt-member" class="info-icon" />
            {{ memberList[0]?.dispatchNo }}
            <span v-if="stream.listen">讲话中</span>
          </div>
        </div>
        <div class="timer">{{ timeTikFormat }}</div>
        <div class="row">
          <div class="info info-btn" @click="requestPttTalk">
            <svg-icon v-if="!startCall" name="ptt-speak" class="info-icon" />
            <svg-icon v-if="startCall" name="ptt-speaking" class="info-icon" />
            <span v-if="!startCall" class="blue">点击讲话</span>
            <span v-if="startCall" class="blue">结束说话</span>
          </div>
          <div class="info info-btn" @click="hangup">
            <svg-icon name="ptt-hangup" class="info-icon" />
            <span class="red">挂断</span>
          </div>
        </div>
      </div>
    </div>
    <!-- 多人调度 -->
    <div v-if="multiple" class="ptt-client ptt-client-multi">
      <div class="ptt-talk">
        <div class="row">
          <!-- 调度成员 -->
          <div
            class="info"
            v-for="item in currentList"
            :key="item.dispatchNo"
            @mouseenter="showRemove(item)"
            @mouseleave="item.showRemove = false"
          >
            <svg-icon :name="item.host ? 'ptt-platform' : 'ptt-member'" class="info-icon" />
            {{ item.host ? sysUserInfo.name : item.name }}
            <span v-if="item.host && stream.speak">讲话中</span>
            <span v-if="!item.host && stream.listen && spokespersonId === item.uid">讲话中</span>
            <i v-if="item.showRemove && memberList.length > 1" class="remove-user el-icon-close" @click="removeUser(item)"></i>
          </div>
        </div>
        <div class="timer">{{ timeTikFormat }}</div>
        <!-- 分页 -->
        <el-pagination
          v-if="participant.length > pageSize"
          class="pagination"
          background
          :current-page="pageIndex"
          :page-size="pageSize"
          :total="total"
          layout="total, prev, pager, next"
          @current-change="onCurrentChange"
        >
        </el-pagination>
        <!-- 操作 -->
        <div class="row row-footer">
          <!-- 添加成员 -->
          <el-button class="add-user" plain type="primary" icon="el-icon-circle-plus-outline" @click="showAddUsers = true">
            添加成员
          </el-button>
          <div class="info info-btn" @click="requestPttTalk">
            <svg-icon v-if="!startCall" name="ptt-speak" class="info-icon" />
            <svg-icon v-if="startCall" name="ptt-speaking" class="info-icon" />
            <span v-if="!startCall" class="blue">点击讲话</span>
            <span v-if="startCall" class="blue">结束说话</span>
          </div>
          <div class="info info-btn" @click="hangup">
            <svg-icon name="ptt-hangup" class="info-icon" />
            <span class="red">挂断</span>
          </div>
        </div>
      </div>
    </div>
    <!-- 添加成员 -->
    <un-participated-list :visible.sync="showAddUsers" @addUsers="calcCurrentPage" />
  </el-dialog>
</template>

<script>
import unParticipatedList from "@/components/pttClient/unParticipatedList.vue";
import WebPtt from "@/components/pttClient/WebPtt";
import { mapActions, mapGetters, mapState } from "vuex";
import _ from "lodash";
import dayjs from "dayjs";
import duration from "dayjs/plugin/duration";
dayjs.extend(duration);

window.PTTClient = null;
window.PTTClientLoginStatus = false;
export default {
  name: "pttClient",
  components: {
    unParticipatedList,
  },
  props: {
    multiple: Boolean,
  },
  data() {
    return {
      userTalking: false,
      stream: {
        speak: false,
        listen: false,
      },
      startCall: false,
      timeTikFormat: "00:00:00",
      timingTimer: null,
      openTime: null,
      timeTik: 0,
      /**
       * 双方都静默时长
       * 1、一建立连接就开始且持续计时(setTimingTimer，每秒+1)
       * 2、有人说话就赋值为0（onSpeak）
       * 3、一直没人讲话就会再持续累加（注意：就算有人说话也只是赋值把silenceTime赋值为0，setTimingTimer定时器没有结束，一直在更新silenceTime）
       * 4、直至双方都静默时长到达一分钟(silenceTime>60)触发挂断（watch silenceTime）
       */
      silenceTime: 0,
      pageIndex: 1,
      pageSize: 16,
      currentList: [],
      showAddUsers: false,
      spokespersonId: null,
    };
  },
  computed: {
    ...mapState({
      pttAccount: (state) => state.gwsd.account,
      memberList: (state) => state.gwsd.memberList,
      sysUserInfo: (state) => state.user.sysUserInfo,
      participant: (state) => state.gwsd.participant,
    }),
    ...mapGetters(["pttGroupId"]),
    total() {
      return this.participant.length;
    },
  },
  watch: {
    // 双方静默1分钟自动挂断
    // silenceTime(val) {
    //   if (val && val > 60) {
    //     this.hangup();
    //   }
    // },
  },
  methods: {
    ...mapActions(["dismissPTTGroup" /*删除组群*/, "removePTTGroupUser" /*删除组群成员*/, "getParticipant"]),
    clearTimer() {
      this.silenceTime = 0;
      this.openTime = null;
      if (this.timingTimer) {
        clearTimeout(this.timingTimer);
        this.timingTimer = null;
      }
    },
    // 挂断，直接触发关闭弹框回调onClose方法
    hangup() {
      if (this.$refs.speak && this.$refs.speak.srcObject) {
        // 结束通话
        window.PTTClient.stopCall();
      }
      this.$emit("close");
      this.$emit("update:visible", false);
    },
    // 关闭弹框回调
    onClose() {
      // 退出组群
      this.dismissPTTGroup({
        adminName: this.pttAccount.adminName,
        gid: this.pttGroupId,
      })
        .then((res) => {
          console.log(res);
          // if (window.PTTClient && window.PTTClientLoginStatus) {
          //   window.PTTClient.leaveGroup();
          //   window.PTTClient.logout();
          // }
          if (this.$refs.speak && this.$refs.speak.srcObject) {
            // 结束通话
            window.PTTClient.stopCall();
          }
        })
        .finally(() => {
          // 结束通话
          this.startCall = false;
        });
      // 清空定时器
      this.clearTimer();
      // 关闭弹框
      this.$emit("close");
      this.$emit("update:visible", false);
    },
    // 弹框打开时回调
    onOpen() {
      this.stream.listen = false;
      this.stream.speak = false;
      this.$emit("open");
      // 计算当前页显示的成员
      this.calcCurrentPage();
      // 初始化PTTClient
      this.initClient();
    },
    // 初始化PTTClient
    initClient() {
      // 窗口打开时判断是否已经生成了客户端
      if (!window.PTTClient) {
        window.PTTClientLoginStatus = false;
        const { account, password } = this.pttAccount; // 调度员PTT账号信息 sapi/misc/virtualAccount/detail 接口获取
        // 创建PTTClient客户端
        window.PTTClient = new WebPtt({
          speak: this.$refs.speak,
          listen: this.$refs.listen,
          account,
          password: password ? password : "111111",
          onLogin: (status, uid) => {
            console.log("ptt login status=", status, " uid=", uid);
            // 登录成功，更新登录状态
            window.PTTClientLoginStatus = Number(status) === 0;
            if (window.PTTClientLoginStatus) {
              // 自动加入组群
              window.PTTClient.joinGroup(this.pttGroupId);
            }
          },
          onLogout: () => {
            window.PTTClientLoginStatus = false;
          },
          onError: (code, info) => {
            if (Number(code) !== 0) {
              console.log("error happen=", code, info);
              // this.$message.error("对讲服务连接失败");
            } else {
              console.log("success no error");
              // this.$message.success("对讲服务连接成功");
              // ws 连接成功后 判断是否登录过，没有就自动登录
              // PTTClientLoginStatus在onLogin回调中更新
              if (!window.PTTClientLoginStatus) window.PTTClient.login();
            }
          },
          onAddStream: (dir, stream) => {
            console.log("add stream", dir, stream);
            // if (dir === 1) {
            //   this.stream.speak = stream;
            // } else {
            //   this.stream.listen = stream;
            // }
          },
          onSpeak: (status, uid) => {
            // 讲话状态回调
            // status < 0 本机讲话失败
            // status == 0 uid == 0 其他成员讲话结束
            // status == 0 uid > 0 其他成员讲话开始，uid 为讲话者 id
            console.log("ptt speak status=", status, " uid=", uid);
            if (status >= 0) {
              this.silenceTime = 0;
              if (uid > 0) {
                this.spokespersonId = uid;
                this.stream.listen = true;
              } else {
                this.stream.listen = false;
              }
            }
          },
        });
      } else {
        // 已经生成了客户端，是二次打开窗口，如果没登录就再登录一次
        if (!window.PTTClientLoginStatus) window.PTTClient.login();
      }
      // 开始计时
      this.openTime = dayjs();
      this.setTimingTimer();
    },
    // 根据participant的total和pageSize,pageIndex计算当前显示的currentList
    calcCurrentPage() {
      const { dispatchNo } = this.sysUserInfo;
      this.getParticipant().then(() => {
        const currentList = this.participant.slice((this.pageIndex - 1) * this.pageSize, this.pageIndex * this.pageSize);
        const list = currentList.map((item) => {
          return {
            ...item,
            showRemove: false,
            host: dispatchNo === item.account,
          };
        });
        // this.currentList按list数组host字段true在前false在后排序
        this.currentList = list.sort((a) => (a.host ? -1 : 1));
        console.log("currentList", this.currentList);
      });
    },
    // 开始or结束通话
    requestPttTalk: _.debounce(
      /*加个防抖避免快速重复点击*/ function () {
        if (window.PTTClient && window.PTTClientLoginStatus) {
          this.startCall = !this.startCall;
          if (this.startCall) {
            window.PTTClient.startCall();
          } else {
            window.PTTClient.stopCall();
            this.silenceTime = 0;
          }
        }
      },
      1000
    ),
    setTimingTimer() {
      this.silenceTime++;
      if (this.stream?.speak || this.stream?.listen) {
        this.silenceTime = 0;
      }
      this.timingTimer = setTimeout(() => {
        this.timeTik = dayjs().diff(this.openTime) || 0;
        const duration = dayjs.duration(this.timeTik);
        const hours = duration.hours() < 10 ? `0${duration.hours()}` : duration.hours();
        const minutes = duration.minutes() < 10 ? `0${duration.minutes()}` : duration.minutes();
        const seconds = duration.seconds() < 10 ? `0${duration.seconds()}` : duration.seconds();
        this.timeTikFormat = hours + ":" + minutes + ":" + seconds;
        this.setTimingTimer();
      }, 1000);
    },
    onCurrentChange(val) {
      this.pageIndex = val;
      this.calcCurrentPage();
    },
    showRemove(item) {
      if (!item.host) item.showRemove = true;
    },
    // 删除组群成员
    removeUser(item) {
      this.removePTTGroupUser(item.account).then((res) => {
        console.log(res);
        this.calcCurrentPage();
      });
    },
  },
  created() {},
  mounted() {},
  beforeDestroy() {
    this.dismissPTTGroup({
      adminName: this.pttAccount.adminName,
      gid: this.pttGroupId,
    }).then((res) => {
      console.log(res);
      // if (window.PTTClient && window.PTTClientLoginStatus) {
      //   window.PTTClient.leaveGroup();
      //   window.PTTClient.logout();
      // }
    });
    this.clearTimer();
  },
};
</script>

<style scoped lang="scss">
::v-deep .ptt-dialog {
  .el-dialog__body {
    padding: 10px;
  }
}
.ptt-client {
  display: flex;
  justify-content: center;
  align-items: center;
  height: 253px;
  width: 100%;
  padding: 16px;
  .ptt-talk {
    height: 100%;
    width: 100%;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    .row {
      width: 100%;
      height: 82px;
      display: grid;
      grid-template-columns: 1fr 1fr;
      grid-gap: 10px;
      .info {
        width: 100%;
        height: 80px;
        display: flex;
        flex-direction: column;
        justify-content: space-between;
        align-items: center;
        position: relative;
        &-btn {
          cursor: pointer;
        }
        &-icon {
          width: 100%;
          height: 51px;
        }
        .blue {
          color: #0297d7;
        }
        .red {
          color: #d9001b;
        }
        .remove-user {
          position: absolute;
          right: 25px;
          top: 0;
          cursor: pointer;
          &:hover {
            font-weight: 700;
          }
        }
      }
      .talking {
        color: #0e70ff;
      }
      .icon-hangup {
        display: block;
        width: 32px;
        height: 32px;
        background: url(~@/assets/images/wfc/挂断.png) no-repeat;
        background-size: 100% 100%;
        cursor: pointer;
        position: absolute;
        top: 25px;
        right: 10px;
      }
    }
    .timer {
      width: 100%;
      display: flex;
      justify-content: center;
      align-items: center;
      font-size: 16px;
      color: #0297d7;
      margin: 16px 0;
    }
    .bottom {
      width: 100%;
      height: 164px;
      background: url(~@/assets/images/wfc/语音关闭-2.png) no-repeat;
      background-size: 100% 100%;
      cursor: pointer;
      &.talking {
        background: url(~@/assets/images/wfc/语音开启.png) no-repeat;
        background-size: 100% 100%;
      }
    }
  }
  .group-info {
    height: 100%;
    width: 248px;
    background: #ffffff;
    border-radius: 3px 3px 3px 3px;
    opacity: 1;
    border: 1px solid #d9d9d9;
    .title {
      width: 100%;
      height: 41px;
      background: #ffffff;
      border-bottom: 1px solid #d9d9d9;
      padding-left: 13px;
      display: flex;
      align-items: center;
    }
    .group-list {
      width: 100%;
      height: calc(100% - 41px);
      overflow-y: auto;
      overflow-x: hidden;
      padding: 3px 6px;
      .group-count {
        width: 100%;
        height: 26px;
        background: #ddebff;
        font-size: 10px;
        font-weight: 400;
        color: #0e70ff;
        display: flex;
        align-items: center;
        padding: 0 13px;
      }
      .group-member {
        width: 100%;
        height: 26px;
        background: #ffffff;
        border-bottom: 1px solid #dcdfe6;
        display: flex;
        justify-content: space-between;
        align-items: center;
        padding: 0 13px;
        font-size: 10px;
        font-weight: 400;
        color: #303133;
        line-height: 14px;
        .talking {
          height: 14px;
          font-size: 10px;
          font-weight: 400;
          color: #ff8503;
          line-height: 14px;
        }
      }
    }
  }
  &-multi {
    padding: 10px;
    .ptt-talk {
      .row {
        display: grid;
        grid-template-columns: repeat(6, 1fr);
        grid-gap: 10px;
        &-footer {
          display: flex;
          align-items: center;
          justify-content: center;
          position: relative;
          .info {
            width: 64px;
            margin: 0 20px;
          }
          .add-user {
            position: absolute;
            left: 0;
            top: 50%;
            transform: translate(0, -50%);
          }
        }
      }
    }
  }
}
</style>
